;###########################################################################
;###########################################################################
; ABOUT SPACE-TRIS:
;
;	This is the main portion of code. It has WinMain and performs all
;	of the management for the game.
;
;		- WinMain()
;		- WndProc()
;		- Game_Init()
;		- Game_Main()
;		- Game_Shutdown()
;
;
;###########################################################################
;###########################################################################


;###########################################################################
;###########################################################################
; THE COMPILER OPTIONS
;###########################################################################
;###########################################################################

	.386
	.model flat, stdcall
	option casemap :none   ; case sensitive

;###########################################################################
;###########################################################################
; THE INCLUDES SECTION
;###########################################################################
;###########################################################################
	
	;================================================
	; These are the Inlcude files for Window calls
	;================================================
	include \masm32\include\windows.inc
	include \masm32\include\comctl32.inc
	include \masm32\include\comdlg32.inc
	include \masm32\include\shell32.inc
	include \masm32\include\user32.inc
	include \masm32\include\kernel32.inc
	include \masm32\include\gdi32.inc

	;====================================
	; The Direct Draw include file
	;====================================
	include Includes\DDraw.inc
	
	;===============================================
	; The Lib's for those included files
	;================================================
	includelib \masm32\lib\comctl32.lib
	includelib \masm32\lib\comdlg32.lib
	includelib \masm32\lib\shell32.lib
	includelib \masm32\lib\gdi32.lib
	includelib \masm32\lib\user32.lib
	includelib \masm32\lib\kernel32.lib

	;=================================================
	; Include the file that has our prototypes
	;=================================================
	include Protos.inc

;###########################################################################
;###########################################################################
; LOCAL MACROS
;###########################################################################
;###########################################################################

	szText MACRO Name, Text:VARARG
		LOCAL lbl
		jmp lbl
		Name db Text,0
		lbl:
	ENDM

	m2m MACRO M1, M2
		push		M2
		pop		M1
	ENDM

	return MACRO arg
		mov	eax, arg
        	ret
	ENDM

	RGB MACRO red, green, blue
		xor	eax,eax
		mov	ah,blue
		shl	eax,8
		mov	ah,green
		mov	al,red
	ENDM

;#################################################################################
;#################################################################################
; Variables we want to use in other modules
;#################################################################################
;#################################################################################

	;===========================================
	; The main window handle
	;===========================================
	PUBLIC	hMainWnd


;#################################################################################
;#################################################################################
; External variables
;#################################################################################
;#################################################################################

	;=================================
	; The DirectDraw stuff
	;=================================
	EXTERN	lpddsprimary	:LPDIRECTDRAWSURFACE4
	EXTERN	lpddsback	:LPDIRECTDRAWSURFACE4


;#################################################################################
;#################################################################################
; BEGIN INITIALIZED DATA
;#################################################################################
;#################################################################################

    .data
	;==============================
	;Text for the Window Title and
	; the class name
	;==============================
    	szDisplayName	db "SPACE-TRIS",0
	szClassName	db "ST_Class",0
	
	;==============================
	;Windows handles and Misc
	;==============================
	msg  MSG	<?>	; For message handling
	CommandLine	dd 0	; for the commandline params
	hMainWnd	dd 0	; Handle to the main window
	hInst		dd 0	; Handle to an Instance

	;========================================================
	; This is used to pass arguments to wvsprintfA it can 
	; hold up to 4 different args
	;========================================================
	;dwArgs		dd 4 dup (0)

	;======================================
	; A value to hold lPitch when locking
	;======================================
	lPitch		dd 0

	;===============================
	; Strings for the Loading bitmap
	;===============================
	szLoading		db "Art\Loading.sfp",0

	;===============================
	; PTR to Loading BMP
	;===============================
	ptr_BMP_LOAD	dd 0


;#################################################################################
;#################################################################################
; BEGIN CONSTANTS
;#################################################################################
;#################################################################################


;#################################################################################
;#################################################################################
; BEGIN EQUATES
;#################################################################################
;#################################################################################

	;=================
	;Utility Equates
	;=================
FALSE		equ	0
TRUE		equ	1

	;=================
	; The Screen BPP
	;=================
screen_bpp	equ	16

	;================
	; resource IDs
	;================
IDI_ICON	equ	01h		


;#################################################################################
;#################################################################################
; BEGIN THE CODE SECTION
;#################################################################################
;#################################################################################

  .code

start:
	;==================================
	; Obtain the instance for the
	; application
	;==================================
	invoke GetModuleHandle, NULL
	mov	hInst, eax

	;==================================
	; Is there a commandline to parse?
	;==================================
	invoke GetCommandLine
	mov	CommandLine, eax

	;==================================
	; Call the WinMain procedure
	;==================================
	invoke WinMain,hInst,NULL,CommandLine,SW_SHOWDEFAULT

	;==================================
	; Leave the program
	;==================================
	invoke ExitProcess,eax


;########################################################################
; WinMain Function
;########################################################################
WinMain proc	hInstance	:DWORD,
		hPrevInst	:DWORD,
		CmdLine		:DWORD,
		CmdShow		:DWORD

	;====================
	; Put LOCALs on stack
	;====================
	LOCAL wc		:WNDCLASS

	;==================================================
	; Fill WNDCLASS structure with required variables
	;==================================================
	mov	wc.style, CS_OWNDC
	mov	wc.lpfnWndProc,offset WndProc
	mov	wc.cbClsExtra,NULL
	mov	wc.cbWndExtra,NULL
	m2m	wc.hInstance,hInst   ;<< NOTE: macro not mnemonic
	invoke GetStockObject, BLACK_BRUSH
	mov	wc.hbrBackground, eax
	mov	wc.lpszMenuName,NULL
	mov	wc.lpszClassName,offset szClassName
	invoke LoadIcon, hInst, IDI_ICON ; icon ID
	mov	wc.hIcon,eax
	invoke LoadCursor,NULL,IDC_ARROW
	mov	wc.hCursor,eax

	;================================
	; Register our class we created
	;================================
	invoke RegisterClass, ADDR wc

	;===========================================
	; Create the main screen
	;===========================================
	invoke CreateWindowEx,NULL,
			ADDR szClassName,
                        ADDR szDisplayName,
                        WS_POPUP or WS_CLIPSIBLINGS or \
			WS_MAXIMIZE or WS_CLIPCHILDREN,
                        0,0,640,480,
                        NULL,NULL,
                        hInst,NULL
        
	;===========================================
	; Put the window handle in for future uses 
	;===========================================
  	mov	hMainWnd, eax

	;====================================
	; Hide the cursor
	;====================================
	invoke ShowCursor, FALSE

	;===========================================
	; Display our Window we created for now
	;===========================================
	invoke ShowWindow, hMainWnd, SW_SHOWDEFAULT

	;=================================
	; Intialize the Game
	;=================================
	invoke Game_Init

	;========================================
	; Check for an error if so leave
	;========================================
	.if eax != TRUE
		jmp	shutdown
	.endif

	;===================================
	; Loop until PostQuitMessage is sent
	;===================================
  	.WHILE TRUE
		invoke	PeekMessage, ADDR msg, NULL, 0, 0, PM_REMOVE
		.if (eax != 0)
			;===================================
			; Break if it was the quit messge
			;===================================
			mov	eax, msg.message
			.IF eax == WM_QUIT
				;======================
				; Break out
				;======================
				jmp	shutdown
			.endif

			;===================================
			; Translate and Dispatch the message
			;===================================
			invoke	TranslateMessage, ADDR msg
			invoke	DispatchMessage, ADDR msg

		.endif

		;================================
		; Call our Main Game Loop 
		;
		; NOTE: This is done every loop
		; iteration no matter what
		;================================
		invoke Game_Main

	.ENDW

shutdown:

	;=================================
	; Shutdown the Game
	;=================================
	invoke Game_Shutdown

	;=================================
	; Show the Cursor
	;=================================
	invoke ShowCursor, TRUE

getout:
	;===========================
	; We are through 
	;===========================
	return msg.wParam

WinMain endp
;########################################################################
; End of WinMain Procedure
;########################################################################



;########################################################################
; Main Window Callback Procedure -- WndProc
;########################################################################
WndProc proc	hWin   :DWORD,
		uMsg   :DWORD,
		wParam :DWORD,
		lParam :DWORD

.if uMsg == WM_COMMAND
	;===========================
	; We don't have a menu, but 
	; if we did this is where it
	; would go!
	;===========================

.elseif uMsg == WM_KEYDOWN
	;=======================================
	; Since we don't have a Direct input
	; system coded yet we will just check 
	; for escape to be pressed
	;=======================================
	mov	eax, wParam
	.if eax == VK_ESCAPE
		;===========================
		; Kill the application
		;===========================
		invoke PostQuitMessage,NULL

	.endif
	
	;==========================
	; We processed it
	;==========================
	return 0 

.elseif uMsg == WM_DESTROY
	;===========================
	; Kill the application
	;===========================
	invoke PostQuitMessage,NULL
 	return 0 
 
.endif

;=================================================
; Let the default procedure handle the message
;=================================================
invoke DefWindowProc,hWin,uMsg,wParam,lParam

ret

WndProc endp
;########################################################################
; End of Main Windows Callback Procedure
;########################################################################




;========================================================================
;========================================================================
; THE GAME PROCEDURES
;========================================================================
;========================================================================


;########################################################################
; Game_Init Procedure
;########################################################################
Game_Init	proc

	;=========================================================
	; This function will setup the game
	;=========================================================
	
	;============================================
	; Initialize Direct Draw -- 640, 480, bpp
	;============================================
	invoke DD_Init, 640, 480, screen_bpp

	;====================================
	; Test for an error
	;====================================
	.if eax == FALSE
		;========================
		; We failed so leave
		;========================
		jmp	err
	
	.endif

	;======================================
	; Read in the bitmap and create buffer
	;======================================
	invoke Create_From_SFP, ADDR ptr_BMP_LOAD, ADDR szLoading, screen_bpp

	;====================================
	; Test for an error
	;====================================
	.if eax == FALSE
		;========================
		; We failed so leave
		;========================
		jmp	err
	
	.endif

	;===================================
	; Lock the DirectDraw back buffer
	;===================================
	invoke DD_Lock_Surface, lpddsback, ADDR lPitch

	;============================
	; Check for an error
	;============================
	.if eax == FALSE
		;===================
		; Jump to err
		;===================
		jmp	err

	.endif

	;===================================
	; Draw the bitmap onto the surface
	;===================================
	invoke Draw_Bitmap, eax, ptr_BMP_LOAD, lPitch, screen_bpp

	;===================================
	; Unlock the back buffer
	;===================================
	invoke DD_Unlock_Surface, lpddsback

	;============================
	; Check for an error
	;============================
	.if eax == FALSE
		;===================
		; Jump to err
		;===================
		jmp	err

	.endif

	;=====================================
	; Everything okay so flip displayed
	; surfaces and make loading visible
	;======================================
	invoke DD_Flip

	;============================
	; Check for an error
	;============================
	.if eax == FALSE
		;===================
		; Jump to err
		;===================
		jmp	err

	.endif

done:
	;===================
	; We completed
	;===================
	return TRUE

err:
	;===================
	; We didn't make it
	;===================
	return FALSE

Game_Init	ENDP
;########################################################################
; END Game_Init
;########################################################################



;########################################################################
; Game_Main Procedure
;########################################################################
Game_Main	proc

	;============================================================
	; This is the heart of the game it gets called over and over
	; and even if we process a message!
	;============================================================



done:
	;===================
	; We completed
	;===================
	return TRUE

err:
	;===================
	; We didn't make it
	;===================
	return FALSE

Game_Main	ENDP
;########################################################################
; END Game_Main
;########################################################################



;########################################################################
; Game_Shutdown Procedure
;########################################################################
Game_Shutdown	proc

	;============================================================
	; This shuts our game down and frees memory we allocated
	;============================================================

	;===========================
	; Shutdown DirectDraw
	;===========================
	invoke DD_ShutDown

	;==========================
	; Free the bitmap memory
	;==========================
	invoke GlobalFree, ptr_BMP_LOAD

done:
	;===================
	; We completed
	;===================
	return TRUE

err:
	;===================
	; We didn't make it
	;===================
	return FALSE

Game_Shutdown	ENDP
;########################################################################
; END Game_Shutdown
;########################################################################

;######################################
; THIS IS THE END OF THE PROGRAM CODE #
;######################################
end start
